(*
    ||M||  This file is part of HELM, an Hypertextual, Electronic        
    ||A||  Library of Mathematics, developed at the Computer Science     
    ||T||  Department, University of Bologna, Italy.                     
    ||I||                                                                
    ||T||  HELM is free software; you can redistribute it and/or         
    ||A||  modify it under the terms of the GNU General Public License   
    \   /  version 2 or (at your option) any later version.              
     \ /   This software is distributed as is, NO WARRANTY.              
      V_______________________________________________________________ *)

module U  = NUri
module UH = U.UriHash
module L  = Log
module E  = Entity
module Z  = Bag

exception ObjectNotFound of Z.message

let hsize = 7000 
let env = UH.create hsize

(* Internal functions *******************************************************)

let get_age = 
   let age = ref 0 in
   fun () -> incr age; !age

let error uri = raise (ObjectNotFound (L.items1 (U.string_of_uri uri)))

(* Interface functions ******************************************************)

let set_entity f (ra, na, uri, b) =
   let age = get_age () in
   let entry = ra, {na with E.n_apix = age}, uri, b in
   UH.add env uri entry; f entry

let get_entity f uri =
   try f (UH.find env uri) with Not_found -> error uri
