(*
    ||M||  This file is part of HELM, an Hypertextual, Electronic        
    ||A||  Library of Mathematics, developed at the Computer Science     
    ||T||  Department, University of Bologna, Italy.                     
    ||I||                                                                
    ||T||  HELM is free software; you can redistribute it and/or         
    ||A||  modify it under the terms of the GNU General Public License   
    \   /  version 2 or (at your option) any later version.              
     \ /   This software is distributed as is, NO WARRANTY.              
      V_______________________________________________________________ *)

module U = NUri
module N = Layer

type uri = U.uri

type id = string (* identifier *)

type name = id * bool (* token, real? *)

type meta = Main     (* main object *)
          | InProp   (* inhabitant of a proposition *)
          | Progress (* uncompleted object *)
	  | Private  (* private global definition *)

type node_attrs = {
   n_name: name option; (* name *)
   n_apix: int;         (* additional position index *)
   n_degr: int;         (* degree *)
   n_sort: int;         (* sort *)
}

type root_attrs = {
   r_meta: meta list;                (* metaliguistic classification *) 
   r_info: (string * string) option; (* metaliguistic annotation: language (defaults to "en-US"), text *)
}

type 'term bind = Abst of 'term (* declaration: domain *)
                | Abbr of 'term (* definition: body    *)
		| Void          (* exclusion           *)

type 'term entity = root_attrs * node_attrs * uri * 'term bind (* attrs, uri, binder *)

(* helpers ******************************************************************)

let node_attrs ?(apix=0) ?name ?(degr=0) ?(sort=0) () = {
   n_apix = apix; n_name = name; n_degr = degr; n_sort = sort
}

let root_attrs ?(meta=[]) ?info () = {
   r_meta = meta; r_info = info;
}

let empty_node = node_attrs ()

let empty_root = root_attrs ()

let common f (ra, na, u, _) = f ra na u

let rec name err f a = match a.n_name with
   | Some (n, r) -> f n r
   | None        -> err ()

let rec info err f a = match a.r_info with
   | Some (lg, tx) -> f lg tx
   | None          -> err ()

let xlate f xlate_term = function
   | ra, na, uri, Abst t ->
      let f t = f (ra, na, uri, Abst t) in xlate_term f t
   | ra, na, uri, Abbr t ->
      let f t = f (ra, na, uri, Abbr t) in xlate_term f t
   | _, _, _, Void       ->
      assert false
