(*
    ||M||  This file is part of HELM, an Hypertextual, Electronic        
    ||A||  Library of Mathematics, developed at the Computer Science     
    ||T||  Department, University of Bologna, Italy.                     
    ||I||                                                                
    ||T||  HELM is free software; you can redistribute it and/or         
    ||A||  modify it under the terms of the GNU General Public License   
    \   /  version 2 or (at your option) any later version.              
     \ /   This software is distributed as is, NO WARRANTY.              
      V_______________________________________________________________ *)

module KF = Filename

module C = Cps

type uri_generator = string -> string

type kernel = V4 | V3 | V0

type manager = Quiet
             | Matita
             | Coq

(* interface functions ******************************************************)

let version_string = "Helena 0.8.2 M (February 2015)"

let stage = ref 3            (* stage *)

let trace = ref 0            (* trace level *)

let summary = ref false      (* log summary information *)

let xdir = ref ""            (* directory for XML output *)

let kernel = ref V3          (* kernel type *)

let si = ref false           (* use sort inclusion *)

let cover = ref ""           (* initial uri segment *)

let cc = ref false           (* activate conversion constraints *)

let expand = ref false       (* always expand global definitions *)

let indexes = ref false      (* show de Bruijn indexes *)

let icm = ref 0              (* complexity measure of relocated terms *)

let unquote = ref false      (* do not quote identifiers when lexing *)

let debug_parser = ref false (* output parser debug information *)

let debug_lexer = ref false  (* output lexer debug information *)

let manager_dir = ref ""     (* output directory for manager *)

let manager = ref Quiet      (* manager *)

let preamble = ref ""        (* preamble file for manager *)

let alpha = ref ""           (* prefix of numeric identifiers *)

let kernel_id () = 
   let id = match !kernel with
      | V4 -> "Environment"
      | V3 -> "Environment_V3"
      | V0 -> "Environment_V0"
   in
   let si = if !si then "_si" else "" in
   id ^ si

let get_baseuri () =
   String.concat "/" ["ld:"; kernel_id (); !cover; "" ]

let get_mk_uri () =
   let bu = get_baseuri () in
   fun s -> KF.concat bu (s ^ ".ld")

let clear () =
   stage := 3; trace := 0; summary := false; 
   xdir := ""; kernel := V3; si := false; cover := ""; 
   expand := false; indexes := false; icm := 0; unquote := false;
   debug_parser := false; debug_lexer := false;
   manager_dir := ""; manager := Quiet
